"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EasyconfigService = void 0;
const dotenv = require("dotenv");
const dotenvExpand = require("dotenv-expand");
const parseEnv_1 = require("./parseEnv");
const fs = require("fs");
const path = require("path");
const common_1 = require("@nestjs/common");
const easyconfig_error_1 = require("./easyconfig.error");
class EasyconfigService {
    constructor(config) {
        this.sampleFile = '.env.sample';
        this.tryGetConfigFromEnv = (config) => {
            const sampleFile = config.sampleFilePath
                ? path.resolve(config.sampleFilePath)
                : this.sampleFile;
            try {
                if (!config.path && process.env.NODE_ENV) {
                    this.envConfig = this.returnEnvs(Object.assign(Object.assign({}, config), { path: path.resolve(`.env.${process.env.NODE_ENV}`) }));
                }
                else if (!config.path && !process.env.NODE_ENV) {
                    throw new Error('Failed to load configs. Either pass file or NODE_ENV :(');
                }
                else {
                    this.envConfig = this.returnEnvs(Object.assign(Object.assign({}, config), { path: path.resolve(config.path) }));
                }
                if (config.safe) {
                    this.safeCheck(Object.keys(this.envConfig), sampleFile);
                }
                this.envConfig = (0, parseEnv_1.default)(this.envConfig, config);
            }
            catch (err) {
                throw new easyconfig_error_1.EasyconfigError(err);
            }
        };
        this.logger = config.logger || new common_1.Logger(EasyconfigService.name);
        this.tryGetConfigFromEnv(config);
    }
    returnEnvs(config) {
        const thisEnv = dotenv.config({
            debug: config.debug,
            encoding: config.encoding,
            path: config.path,
        });
        if (config.expand) {
            return dotenvExpand.expand(thisEnv).parsed;
        }
        return thisEnv.parsed;
    }
    get(key) {
        const configExists = key in this.envConfig;
        if (!configExists) {
            this.logger.warn('The key was not found in config file 😕');
            return;
        }
        return this.envConfig[key];
    }
    safeCheck(userEnvFile, config) {
        const src = Object.keys(dotenv.parse(fs.readFileSync(path.resolve(config))));
        const missingKeys = src
            .filter(x => !userEnvFile.includes(x))
            .concat(userEnvFile.filter(x => !src.includes(x)));
        if (missingKeys.length !== 0) {
            this.logger
                .error(`MissingEnvVarsError: ${missingKeys} were defined in .env.example but are not present in the environment:
        This may cause the app to misbehave.`);
        }
        else {
            this.logger.debug('Config looks good :) ');
        }
    }
}
exports.EasyconfigService = EasyconfigService;
