"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const DEFAULT_OPTIONS = {
    assignToProcessEnv: true,
    overrideProcessEnv: false,
    parseLog: false,
    ignoreFunctions: true,
};
function logger(msg, parseLog) {
    if (parseLog) {
        common_1.Logger.log(msg, 'ParseService');
    }
}
function isStringBlank(str) {
    const l = str.length;
    let a;
    for (let i = 0; i < l; i++) {
        a = str.charCodeAt(i);
        if ((a < 9 || a > 13) &&
            a !== 32 &&
            a !== 133 &&
            a !== 160 &&
            a !== 5760 &&
            a !== 6158 &&
            (a < 8192 || a > 8205) &&
            a !== 8232 &&
            a !== 8233 &&
            a !== 8239 &&
            a !== 8287 &&
            a !== 8288 &&
            a !== 12288 &&
            a !== 65279) {
            return false;
        }
    }
    return true;
}
function isSANB(val) {
    return typeof val === 'string' && !isStringBlank(val);
}
exports.default = (env, options = DEFAULT_OPTIONS) => {
    const envOptions = Object.assign(Object.assign({}, DEFAULT_OPTIONS), (options || {}));
    const parsed = {};
    for (const key of Object.keys(env)) {
        if (env[key]) {
            if (envOptions.ignoreFunctions && typeof env[key] === 'function') {
                continue;
            }
            parsed[key] = parseKey(env[key], key, envOptions);
            if (envOptions.assignToProcessEnv === true) {
                if (envOptions.overrideProcessEnv === true) {
                    process.env[key] = parsed[key] || process.env[key];
                }
                else {
                    process.env[key] = process.env[key] || parsed[key];
                }
            }
        }
    }
    return parsed;
};
function parseKey(value, key, options) {
    logger(`parsing key ${key} with value ${value}`, options.parseLog);
    if (value.toString().indexOf('`') === 0 &&
        value.toString().lastIndexOf('`') === value.toString().length - 1) {
        logger(`key ${key} is wrapped in bacticks and will be ignored from parsing`, options.parseLog);
        return value.toString().substring(1, value.toString().length - 1);
    }
    if (value.toString().lastIndexOf('*') === value.toString().length - 1 &&
        value.toString().indexOf(',') === -1) {
        logger(`key ${key} ended in * and will be ignored from parsing`, options.parseLog);
        return value.toString().substring(0, value.toString().length - 1);
    }
    if (value.toString().toLowerCase() === 'true' ||
        value.toString().toLowerCase() === 'false') {
        logger(`key ${key} parsed as a Boolean`, options.parseLog);
        return value.toString().toLowerCase() === 'true';
    }
    if (isSANB(value) && !Number.isNaN(Number(value))) {
        return Number(value);
    }
    if (typeof value.includes === 'function' && value.includes(',')) {
        return value
            .split(',')
            .filter(function (str) {
            return str !== '';
        })
            .map(parseKey);
    }
    return value;
}
